/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JMXHelperService.java 19402 2010-03-10 13:26:25Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.admin.jdbcconnections.service;

import java.io.IOException;

import javax.management.Attribute;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.InvalidAttributeValueException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.ow2.jonas.lib.management.extensions.manager.ManagementEntryPoint;

/**
 * JMX Helper class in order to get a connection to the JMX server.
 *
 * @author Florent Benoit
 */
public abstract class JMXHelperService {

    /**
     * Connection to the JOnAS server.
     */
    private MBeanServerConnection mBeanServerConnection = null;

    /**
     * Domain name.
     */
    private String domainName = null;

    /**
     * Default constructor.
     */
    public JMXHelperService() {
        this.domainName = ManagementEntryPoint.getInstance().getDomainName();
        this.mBeanServerConnection = ManagementEntryPoint.getInstance().getServerConnection();
    }

    /**
     * @return the JMX connection to the MBean server
     */
    public MBeanServerConnection getMBeanServerConnection() {
        return mBeanServerConnection;
    }

    /**
     * @return the domain name.
     */
    public String getDomainName() {
        return domainName;
    }

    /**
     * @param objectName the stringified ObjectName
     * @return The JMX ObjectName built from the given string
     */
    protected ObjectName buildObjectName(final String objectName) {
        try {
            return ObjectName.getInstance(objectName);
        } catch (MalformedObjectNameException e) {
            throw new IllegalStateException("Cannot build ObjectName from '" + objectName + "'.");
        } catch (NullPointerException e) {
            throw new IllegalStateException("Cannot build ObjectName from '" + objectName + "'.");
        }
    }

    /**
     * @param oName the given ObjectName
     * @return true if the given objectname is registered, else false
     */
    protected boolean isRegistered(final ObjectName oName) {
        try {
            return mBeanServerConnection.isRegistered(oName);
        } catch (IOException e) {
            throw new IllegalStateException("Cannot check if ObjectName '" + oName + "' is registered");
        }
    }

    /**
     * Update the given JMX attribute on the given ObjectName.
     *
     * @param objectName the ObjectName to update
     * @param attribute the attribute to update
     */
    protected void setAttribute(final ObjectName objectName, final Attribute attribute) {
        try {
            mBeanServerConnection.setAttribute(objectName, attribute);
        } catch (InstanceNotFoundException e) {
            throw new IllegalStateException("Cannot set a new value for the attribute name '" + attribute.getName()
                + "' with value '" + attribute.getValue() + "' on ObjectName '" + objectName + "'.", e);
        } catch (AttributeNotFoundException e) {
            throw new IllegalStateException("Cannot set a new value for the attribute name '" + attribute.getName()
                + "' with value '" + attribute.getValue() + "' on ObjectName '" + objectName + "'.", e);
        } catch (InvalidAttributeValueException e) {
            throw new IllegalStateException("Cannot set a new value for the attribute name '" + attribute.getName()
                + "' with value '" + attribute.getValue() + "' on ObjectName '" + objectName + "'.", e);
        } catch (MBeanException e) {
            throw new IllegalStateException("Cannot set a new value for the attribute name '" + attribute.getName()
                + "' with value '" + attribute.getValue() + "' on ObjectName '" + objectName + "'.", e);
        } catch (ReflectionException e) {
            throw new IllegalStateException("Cannot set a new value for the attribute name '" + attribute.getName()
                + "' with value '" + attribute.getValue() + "' on ObjectName '" + objectName + "'.", e);
        } catch (IOException e) {
            throw new IllegalStateException("Cannot set a new value for the attribute name '" + attribute.getName()
                + "' with value '" + attribute.getValue() + "' on ObjectName '" + objectName + "'.", e);
        }
    }

    /**
     * Allows to get an attribute value
     *
     * @param <T> the type of the object to get/cast
     * @param objectName the objectname from which we want the attribute's value
     * @param attributeName the name of the attribute
     * @return the attribute's value
     */
    @SuppressWarnings("unchecked")
    protected <T> T getAttribute(final ObjectName objectName, final String attributeName) {
        try {
            return (T) mBeanServerConnection.getAttribute(objectName, attributeName);
        } catch (AttributeNotFoundException e) {
            throw new IllegalStateException(
                "Cannot get attribute " + attributeName + "' from ObjectName '" + objectName + "'.", e);
        } catch (InstanceNotFoundException e) {
            throw new IllegalStateException(
                "Cannot get attribute " + attributeName + "' from ObjectName '" + objectName + "'.", e);
        } catch (MBeanException e) {
            throw new IllegalStateException(
                "Cannot get attribute " + attributeName + "' from ObjectName '" + objectName + "'.", e);
        } catch (ReflectionException e) {
            throw new IllegalStateException(
                "Cannot get attribute " + attributeName + "' from ObjectName '" + objectName + "'.", e);
        } catch (IOException e) {
            throw new IllegalStateException(
                "Cannot get attribute " + attributeName + "' from ObjectName '" + objectName + "'.", e);
        }
    }

    /**
     * Invoke the given operation with the specified parameters.
     *
     * @param <T> the type of the result
     * @param objectName the given object name
     * @param operationName the name of the operation
     * @param parameters the parameters of the operation
     * @param signature the signature of the operation
     * @return the result of the invocation
     */
    @SuppressWarnings("unchecked")
    protected <T> T invoke(final ObjectName objectName, final String operationName, final Object[] parameters,
        final String[] signature) {
        try {
            return (T) getMBeanServerConnection().invoke(objectName, operationName, parameters, signature);
        } catch (InstanceNotFoundException e) {
            throw new IllegalStateException("Cannot invoke operation " + operationName + "' on ObjectName '" + objectName
                + "' with signature '" + signature + "'", e);
        } catch (MBeanException e) {
            throw new IllegalStateException("Cannot invoke operation " + operationName + "' on ObjectName '" + objectName
                + "' with signature '" + signature + "'", e);
        } catch (ReflectionException e) {
            throw new IllegalStateException("Cannot invoke operation " + operationName + "' on ObjectName '" + objectName
                + "' with signature '" + signature + "'", e);
        } catch (IOException e) {
            throw new IllegalStateException("Cannot invoke operation " + operationName + "' on ObjectName '" + objectName
                + "' with signature '" + signature + "'", e);
        }
    }

}
